# Copyright (C) 2006 Ulrik Sverdrup
#
#   dragbox -- Commandline drag-and-drop tool for GNOME
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

try:
	import dbus
	import dbus.service
	if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
		import dbus.glib
except:
	print "Dbus not available"
    
object_path = "/org/gna/dragbox"
server_interface = "org.gna.DragboxIFace"
service_name = "org.gna.dragbox"

class server(dbus.service.Object):
	""" 
	Provides a dbus server controller
	Interface methods:
	* ping
	* send_files    
	* send_texts
	 
	""" 
	def __init__(self, bus_name, object_path=object_path):
		dbus.service.Object.__init__(self, bus_name, object_path)
		self.parent = None

	def set_delegate(self, delegate):
		""" 
		set the delegate of the server 
		@param delegate: The object receiving callbacks 
		@type delegate: an object.  
		""" 
		self.parent = delegate
	
	@dbus.service.method(server_interface)
	def send_file(self, f):
		""" 
		dbus service method
		remember to send absolute paths!
		@param data: List of files to send to server parent 
		@type data: a string object.   
		 
		""" 
		self.send_files([f])
		
	@dbus.service.method(server_interface)
	def send_files(self, data):
		""" 
		dbus service method
		remember to send absolute paths!
		@param data: List of files to send to server parent 
		@type data: a list object.   
		 
		""" 
		if self.parent: self.parent.add_files(data)
	
	@dbus.service.method(server_interface)
	def send_text(self, t):
		""" 
		dbus service method
		@param data: A string to send to server parent 
		@type data: a string object.   
		 
		""" 
		self.send_texts([t])
			
	@dbus.service.method(server_interface)
	def send_texts(self, data):
		""" 
		dbus service method
		@param data: List of texts to send to server parent 
		@type data: a list object.   
		 
		""" 
		if self.parent: self.parent.add_texts(data)
	
	@dbus.service.method(server_interface)
	def ping(self):
		""" 
		dbus service method 
		Tests the dbus connection
		Sends activate_application 
		 
		""" 
		if self.parent:
			self.parent.activate_application() 
			return True
		return False 


def make_server():
	""" 
	Returns a server instance inited with correct service
	information 
	 
	""" 
	session_bus = dbus.SessionBus()
	bus_name = dbus.service.BusName(service_name, bus=session_bus)
	return server(bus_name) #instantiation!! important


class client(object):
	u"""
	Tries to connect to previously started server
	
	"""
	def __init__(self): 
		bus = dbus.SessionBus()
		dbusobj = bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')

		# Check with dbus if service is available
		dbusiface = dbus.Interface(dbusobj, 'org.freedesktop.DBus')
		if dbusiface.NameHasOwner(service_name):
				proxy_obj = bus.get_object(service_name, object_path)
				#print proxy_obj
				self.iface = dbus.Interface(proxy_obj, server_interface)
				self.iface.ping()
		else:
			self.iface = None
			#print "No running instance found"
			raise
			

	def get_interface(self):
		""" 
		Returns the dbus interface to the "server" 
		""" 
		return self.iface 

	 
